Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

USE work.pkg_081006.all;

----------------------------------------------------
-- This module provides timing and control for the
-- Booster BLM Integrator control.

entity AD7654_Intf_v4 is
port(
		clk			: in std_logic;
		nreset		: in std_logic;
		Get_ADC		: in std_logic;
		ADC_SCLK	: in std_logic;
		ADC_BUSY	: in std_logic;
		ADC_SYNC	: in std_logic;
		ADC_DATA	: in std_logic;
		
		ADC_CONVn	: out std_logic;
		ADC_A0		: out std_logic;
		Data_Strb	: out std_logic;
		
		ADC_A_Data	: out std_logic_vector(15 downto 0);
		ADC_B_Data	: out std_logic_vector(15 downto 0)
		
);
end AD7654_Intf_v4;

--########################################################
--				A R C H I T E C T U R E
--########################################################
architecture main of AD7654_Intf_v4 is

-- Sync Inputs -------------------------------------------
	signal convst		: std_logic;
	
	signal sync_latch	: std_logic_vector(2 downto 0);
	signal latch_data	: std_logic;
	
	signal sync_strb_out: std_logic_vector(3 downto 0);
	signal strb_out		: std_logic;

-- Input Shift Register ----------------------------------
	signal sr			: std_logic_vector(31 downto 0);

-- Output Data Buffers -----------------------------------
	signal A_Data		: std_logic_vector(15 downto 0);
	signal B_Data		: std_logic_vector(15 downto 0);

--========================================================
--						B E G I N
--========================================================
begin

SYNC_CONVST: led 
port map(sig_in=>Get_ADC, clk=>clk,	clrn=>nreset, pls_out=>convst); 

----------------------------------------------------------
INPUT_SHIFT_REG:
process(clk, nreset, ADC_SCLK, ADC_SYNC, ADC_DATA, sr)
begin
	if(nreset='0') 					then sr <= (others => '0');		
	elsif(rising_edge(ADC_SCLK))	then 
		if 	 (ADC_SYNC = '1') then 	
			sr(0) <= ADC_DATA;
			sr(31 downto 1) <= sr(30 downto 0);
		end if;
	end if;
end process;
		
----------------------------------------------------------
SYNC_LATCH_DATA: -- syncronize the falling edge of ADC_SYNC
process(clk, nreset, ADC_SYNC)
begin
	if   (nreset = '0')		then	sync_latch <= (others =>'1');
	elsif(rising_edge(clk))	then 
		sync_latch(0) 	<= not ADC_SYNC;
		sync_latch(1) 	<= sync_latch(0);
		sync_latch(2) 	<= sync_latch(1);
		
		latch_data 		<= sync_latch(1) and not sync_latch(2);
		
	end if;
end process;

----------------------------------------------------------
SYNC_STROBE_OUT: -- syncronize the falling edge of ADC_BUSY
process(clk, nreset, ADC_BUSY)
begin
	if   (nreset = '0')		then	sync_strb_out <= (others =>'1');
	elsif(rising_edge(clk))	then 
		sync_strb_out(0) <= not ADC_BUSY;
		sync_strb_out(1) <= sync_strb_out(0);
		sync_strb_out(2) <= sync_strb_out(1);
		sync_strb_out(3) <= sync_strb_out(2);
		
		strb_out		<= sync_strb_out(2) and not sync_strb_out(3);
		
	end if;
end process;

----------------------------------------------------------
DATA_OUTPUT_BUFFERS:
process(clk, nreset, latch_data)
begin
	if   (nreset = '0')		then	
		A_Data <= (others =>'0');
		B_Data <= (others =>'0');
	elsif(rising_edge(clk))	then 
		if (latch_data = '1') then 	
			A_Data <= sr(31 downto 16);
			B_Data <= sr(15 downto 0);
		end if;
	end if;
end process;

ADC_CONVn 	<= NOT convst;
ADC_A0		<= '0';
ADC_A_Data	<= A_Data;
ADC_B_Data	<= B_Data;

Data_Strb	<= strb_out;
	
end;             

